@@ -1,10 +1,10 @@
 /* chown-core.c -- core functions for changing ownership.
-   Copyright (C) 2000, 2002, 2003, 2004, 2005 Free Software Foundation.
+   Copyright (C) 2000, 2002-2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Extracted from chown.c/chgrp.c and librarified by Jim Meyering.  */
 
@@ -27,11 +26,16 @@
 #include "chown-core.h"
 #include "error.h"
 #include "inttostr.h"
-#include "lchown.h"
 #include "quote.h"
 #include "root-dev-ino.h"
 #include "xfts.h"
 
+#define FTSENT_IS_DIRECTORY(E)	\
+  ((E)->fts_info == FTS_D	\
+   || (E)->fts_info == FTS_DC	\
+   || (E)->fts_info == FTS_DP	\
+   || (E)->fts_info == FTS_DNR)
+
 enum RCH_status
   {
     /* we called fchown and close, and both succeeded */
@@ -43,6 +47,10 @@ enum RCH_status
     /* SAME_INODE check failed */
     RC_inode_changed,
 
+    /* open/fchown isn't needed, isn't safe, or doesn't work due to
+       permissions problems; fall back on chown */
+    RC_do_ordinary_chown,
+
     /* open, fstat, fchown, or close failed */
     RC_error
   };
@@ -159,10 +167,7 @@ describe_change (const char *file, enum Change_status changed,
 
 /* Change the owner and/or group of the FILE to UID and/or GID (safely)
    only if REQUIRED_UID and REQUIRED_GID match the owner and group IDs
-   of FILE.  ORIG_ST must be the result of `stat'ing FILE.  If this
-   function ends up being called with a FILE that is a symlink and when
-   we are *not* dereferencing symlinks, we'll detect the problem via
-   the SAME_INODE test below.
+   of FILE.  ORIG_ST must be the result of `stat'ing FILE.
 
    The `safely' part above means that we can't simply use chown(2),
    since FILE might be replaced with some other file between the time
@@ -172,49 +177,46 @@ describe_change (const char *file, enum Change_status changed,
    the preceding stat call, and only then, if appropriate (given the
    required_uid and required_gid constraints) do we call fchown.
 
-   A minor problem:
-   This function fails when FILE cannot be opened, but chown/lchown have
-   no such limitation.  But this may not be a problem for chown(1),
-   since chown is useful mainly to root, and since root seems to have
-   no problem opening `unreadable' files (on Linux).  However, this can
-   cause trouble when non-root users apply chgrp to files they own but
-   to which they have neither read nor write access.  For now, that
-   isn't a problem since chgrp doesn't have a --from=O:G option.
+   Return RC_do_ordinary_chown if we can't open FILE, or if FILE is a
+   special file that might have undesirable side effects when opening.
+   In this case the caller can use the less-safe ordinary chown.
 
    Return one of the RCH_status values.  */
 
 static enum RCH_status
-restricted_chown (char const *file,
+restricted_chown (int cwd_fd, char const *file,
 		  struct stat const *orig_st,
 		  uid_t uid, gid_t gid,
 		  uid_t required_uid, gid_t required_gid)
 {
   enum RCH_status status = RC_ok;
   struct stat st;
-  int o_flags = (O_NONBLOCK | O_NOCTTY);
+  int open_flags = O_NONBLOCK | O_NOCTTY;
+  int fd;
 
-  int fd = open (file, O_RDONLY | o_flags);
-  if (fd < 0)
+  if (required_uid == (uid_t) -1 && required_gid == (gid_t) -1)
+    return RC_do_ordinary_chown;
+
+  if (! S_ISREG (orig_st->st_mode))
     {
-      fd = open (file, O_WRONLY | o_flags);
-      if (fd < 0)
-	return RC_error;
+      if (S_ISDIR (orig_st->st_mode))
+	open_flags |= O_DIRECTORY;
+      else
+	return RC_do_ordinary_chown;
     }
 
+  fd = openat (cwd_fd, file, O_RDONLY | open_flags);
+  if (! (0 <= fd
+	 || (errno == EACCES && S_ISREG (orig_st->st_mode)
+	     && 0 <= (fd = openat (cwd_fd, file, O_WRONLY | open_flags)))))
+    return (errno == EACCES ? RC_do_ordinary_chown : RC_error);
+
   if (fstat (fd, &st) != 0)
-    {
-      status = RC_error;
-      goto Lose;
-    }
-
-  if ( ! SAME_INODE (*orig_st, st))
-    {
-      status = RC_inode_changed;
-      goto Lose;
-    }
-
-  if ((required_uid == (uid_t) -1 || required_uid == st.st_uid)
-      && (required_gid == (gid_t) -1 || required_gid == st.st_gid))
+    status = RC_error;
+  else if (! SAME_INODE (*orig_st, st))
+    status = RC_inode_changed;
+  else if ((required_uid == (uid_t) -1 || required_uid == st.st_uid)
+	   && (required_gid == (gid_t) -1 || required_gid == st.st_gid))
     {
       if (fchown (fd, uid, gid) == 0)
 	{
@@ -228,7 +230,6 @@ restricted_chown (char const *file,
 	}
     }
 
- Lose:
   { /* FIXME: remove these curly braces when we assume C99.  */
     int saved_errno = errno;
     close (fd);
@@ -261,7 +262,20 @@ change_file_owner (FTS *fts, FTSENT *ent,
     {
     case FTS_D:
       if (chopt->recurse)
-	return true;
+	{
+	  if (ROOT_DEV_INO_CHECK (chopt->root_dev_ino, ent->fts_statp))
+	    {
+	      /* This happens e.g., with "chown -R --preserve-root 0 /"
+		 and with "chown -RH --preserve-root 0 symlink-to-root".  */
+	      ROOT_DEV_INO_WARN (file_full_name);
+	      /* Tell fts not to traverse into this hierarchy.  */
+	      fts_set (fts, ent, FTS_SKIP);
+	      /* Ensure that we do not process "/" on the second visit.  */
+	      ent = fts_read (fts);
+	      return false;
+	    }
+	  return true;
+	}
       break;
 
     case FTS_DP:
@@ -270,6 +284,19 @@ change_file_owner (FTS *fts, FTSENT *ent,
       break;
 
     case FTS_NS:
+      /* For a top-level file or directory, this FTS_NS (stat failed)
+	 indicator is determined at the time of the initial fts_open call.
+	 With programs like chmod, chown, and chgrp, that modify
+	 permissions, it is possible that the file in question is
+	 accessible when control reaches this point.  So, if this is
+	 the first time we've seen the FTS_NS for this file, tell
+	 fts_read to stat it "again".  */
+      if (ent->fts_level == 0 && ent->fts_number == 0)
+	{
+	  ent->fts_number = 1;
+	  fts_set (fts, ent, FTS_AGAIN);
+	  return true;
+	}
       error (0, ent->fts_errno, _("cannot access %s"), quote (file_full_name));
       ok = false;
       break;
@@ -295,7 +322,9 @@ change_file_owner (FTS *fts, FTSENT *ent,
       file_stats = NULL;
     }
   else if (required_uid == (uid_t) -1 && required_gid == (gid_t) -1
-	   && chopt->verbosity == V_off && ! chopt->root_dev_ino)
+	   && chopt->verbosity == V_off
+	   && ! chopt->root_dev_ino
+	   && ! chopt->affect_symlink_referent)
     {
       do_chown = true;
       file_stats = ent->fts_statp;
@@ -306,9 +335,9 @@ change_file_owner (FTS *fts, FTSENT *ent,
 
       /* If this is a symlink and we're dereferencing them,
 	 stat it to get info on the referent.  */
-      if (S_ISLNK (file_stats->st_mode) && chopt->affect_symlink_referent)
+      if (chopt->affect_symlink_referent && S_ISLNK (file_stats->st_mode))
 	{
-	  if (stat (file, &stat_buf) != 0)
+	  if (fstatat (fts->fts_cwd_fd, file, &stat_buf, 0) != 0)
 	    {
 	      error (0, errno, _("cannot dereference %s"),
 		     quote (file_full_name));
@@ -325,17 +354,20 @@ change_file_owner (FTS *fts, FTSENT *ent,
 		      || required_gid == file_stats->st_gid));
     }
 
-  if (do_chown && ROOT_DEV_INO_CHECK (chopt->root_dev_ino, file_stats))
+  /* This happens when chown -LR --preserve-root encounters a symlink-to-/.  */
+  if (ok
+      && FTSENT_IS_DIRECTORY (ent)
+      && ROOT_DEV_INO_CHECK (chopt->root_dev_ino, file_stats))
     {
       ROOT_DEV_INO_WARN (file_full_name);
-      ok = do_chown = false;
+      return false;
     }
 
   if (do_chown)
     {
       if ( ! chopt->affect_symlink_referent)
 	{
-	  ok = (lchown (file, uid, gid) == 0);
+	  ok = (lchownat (fts->fts_cwd_fd, file, uid, gid) == 0);
 
 	  /* Ignore any error due to lack of support; POSIX requires
 	     this behavior for top-level symbolic links with -h, and
@@ -348,43 +380,41 @@ change_file_owner (FTS *fts, FTSENT *ent,
 	}
       else
 	{
-	  if ( required_uid == (uid_t) -1 && required_gid == (gid_t) -1)
+	  /* If possible, avoid a race condition with --from=O:G and without the
+	     (-h) --no-dereference option.  If fts's stat call determined
+	     that the uid/gid of FILE matched the --from=O:G-selected
+	     owner and group IDs, blindly using chown(2) here could lead
+	     chown(1) or chgrp(1) mistakenly to dereference a *symlink*
+	     to an arbitrary file that an attacker had moved into the
+	     place of FILE during the window between the stat and
+	     chown(2) calls.  If FILE is a regular file or a directory
+	     that can be opened, this race condition can be avoided safely.  */
+
+	  enum RCH_status err
+	    = restricted_chown (fts->fts_cwd_fd, file, file_stats, uid, gid,
+				required_uid, required_gid);
+	  switch (err)
 	    {
-	      ok = (chown (file, uid, gid) == 0);
-	    }
-	  else
-	    {
-	      /* Avoid a race condition with --from=O:G and without the
-		 (-h) --no-dereference option.  If fts' stat call determined
-		 that the uid/gid of FILE matched the --from=O:G-selected
-		 owner and group IDs, blindly using chown(2) here could lead
-		 chown(1) or chgrp(1) mistakenly to dereference a *symlink*
-		 to an arbitrary file that an attacker had moved into the
-		 place of FILE during the window between the stat and
-		 chown(2) calls. */
-	      enum RCH_status err
-		= restricted_chown (file, file_stats, uid, gid,
-				    required_uid, required_gid);
-	      switch (err)
-		{
-		case RC_ok:
-		  ok = true;
-		  break;
+	    case RC_ok:
+	      break;
 
-		case RC_error:
-		  ok = false;
-		  break;
+	    case RC_do_ordinary_chown:
+	      ok = (chownat (fts->fts_cwd_fd, file, uid, gid) == 0);
+	      break;
 
-		case RC_inode_changed:
-		  /* FIXME: give a diagnostic in this case?  */
-		case RC_excluded:
-		  do_chown = false;
-		  ok = false;
-		  goto Skip_chown;
+	    case RC_error:
+	      ok = false;
+	      break;
 
-		default:
-		  abort ();
-		}
+	    case RC_inode_changed:
+	      /* FIXME: give a diagnostic in this case?  */
+	    case RC_excluded:
+	      do_chown = false;
+	      ok = false;
+	      break;
+
+	    default:
+	      abort ();
 	    }
 	}
 
@@ -395,15 +425,13 @@ change_file_owner (FTS *fts, FTSENT *ent,
 	 by some other user and operating on files in a directory
 	 where M has write access.  */
 
-      if (!ok && ! chopt->force_silent)
+      if (do_chown && !ok && ! chopt->force_silent)
 	error (0, errno, (uid != (uid_t) -1
 			  ? _("changing ownership of %s")
 			  : _("changing group of %s")),
 	       quote (file_full_name));
     }
 
- Skip_chown:;
-
   if (chopt->verbosity != V_off)
     {
       bool changed =
@@ -448,7 +476,8 @@ chown_files (char **files, int bit_flags,
 
   /* Use lstat and stat only if they're needed.  */
   int stat_flags = ((required_uid != (uid_t) -1 || required_gid != (gid_t) -1
-		     || chopt->verbosity != V_off || chopt->root_dev_ino)
+		     || chopt->affect_symlink_referent
+		     || chopt->verbosity != V_off)
 		    ? 0
 		    : FTS_NOSTAT);
 
