@@ -1,10 +1,11 @@
 /* echo.c, derived from code echo.c in Bash.
-   Copyright (C) 87,89, 1991-1997, 1999-2005 Free Software Foundation, Inc.
+   Copyright (C) 87,89, 1991-1997, 1999-2005, 2007-2008 Free Software
+   Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <config.h>
 #include <stdio.h>
@@ -24,7 +24,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "echo"
 
-#define AUTHORS "FIXME unknown"
+#define AUTHORS proper_name ("FIXME unknown")
 
 /* echo [-neE] [arg ...]
 Output the ARGs.  If -n is specified, the trailing newline is
@@ -95,7 +95,7 @@ If -e is in effect, the following sequences are recognized:\n\
   \\v     vertical tab\n\
 "), stdout);
       printf (USAGE_BUILTIN_WARNING, PROGRAM_NAME);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -142,7 +142,7 @@ main (int argc, char **argv)
   atexit (close_stdout);
 
   if (allow_options)
-    parse_long_options (argc, argv, PROGRAM_NAME, GNU_PACKAGE, VERSION,
+    parse_long_options (argc, argv, PROGRAM_NAME, PACKAGE_NAME, VERSION,
 			usage, AUTHORS, (char const *) NULL);
 
   --argc;
@@ -218,12 +218,12 @@ just_echo:
 		    case 'x':
 		      {
 			unsigned char ch = *s;
-			if (! ISXDIGIT (ch))
+			if (! isxdigit (ch))
 			  goto not_an_escape;
 			s++;
 			c = hextobin (ch);
 			ch = *s;
-			if (ISXDIGIT (ch))
+			if (isxdigit (ch))
 			  {
 			    s++;
 			    c = c * 16 + hextobin (ch);
