@@ -1,10 +1,10 @@
 /* nl -- number lines of files
-   Copyright (C) 89, 92, 1995-2006 Free Software Foundation, Inc.
+   Copyright (C) 89, 92, 1995-2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -12,8 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software Foundation,
-   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* Written by Scott Bartram (nancy!scott@uunet.uu.net)
    Revised by David MacKenzie (djm@gnu.ai.mit.edu) */
@@ -36,7 +35,9 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "nl"
 
-#define AUTHORS "Scott Bartram", "David MacKenzie"
+#define AUTHORS \
+  proper_name ("Scott Bartram"), \
+  proper_name ("David MacKenzie")
 
 /* Line-number formats.  They are given an int width, an intmax_t
    value, and a string separator.  */
@@ -217,7 +218,7 @@ second character implies :.  Type \\\\ for \\.  STYLE is one of:\n\
   t         number only nonempty lines\n\
   n         number no lines\n\
   pBRE      number only lines that contain a match for the basic regular\n\
-              expression, BRE\n\
+            expression, BRE\n\
 \n\
 FORMAT is one of:\n\
 \n\
@@ -226,7 +227,7 @@ FORMAT is one of:\n\
   rz   right justified, leading zeros\n\
 \n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
   exit (status);
 }
@@ -348,7 +349,7 @@ proc_text (void)
       break;
     case 'p':
       switch (re_search (current_regex, line_buf.buffer, line_buf.length - 1,
-			 0, line_buf.length - 1, (struct re_registers *) 0))
+			 0, line_buf.length - 1, NULL))
 	{
 	case -2:
 	  error (EXIT_FAILURE, errno, _("error in regular expression search"));
