@@ -1,11 +1,11 @@
 /* Shuffle lines of text.
 
-   Copyright (C) 2006 Free Software Foundation, Inc.
+   Copyright (C) 2006, 2007-2008 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -13,8 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
    Written by Paul Eggert.  */
 
@@ -35,7 +34,7 @@
 /* The official name of this program (e.g., no `g' prefix).  */
 #define PROGRAM_NAME "shuf"
 
-#define AUTHORS "Paul Eggert"
+#define AUTHORS proper_name ("Paul Eggert")
 
 /* The name this program was run with. */
 char *program_name;
@@ -75,7 +74,7 @@ Mandatory arguments to long options are mandatory for short options too.\n\
 \n\
 With no FILE, or when FILE is -, read standard input.\n\
 "), stdout);
-      printf (_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
+      emit_bug_reporting_address ();
     }
 
   exit (status);
@@ -251,6 +250,7 @@ main (int argc, char **argv)
   char const *outfile = NULL;
   char *random_source = NULL;
   char eolbyte = '\n';
+  char **input_lines = NULL;
 
   int optc;
   int n_operands;
@@ -258,7 +258,7 @@ main (int argc, char **argv)
   size_t n_lines;
   char **line;
   struct randint_source *randint_source;
-  size_t const *permutation;
+  size_t *permutation;
 
   initialize_main (&argc, &argv);
   program_name = argv[0];
@@ -366,8 +366,6 @@ main (int argc, char **argv)
     }
   else
     {
-      char **input_lines;
-
       switch (n_operands)
 	{
 	case 0:
@@ -408,5 +406,15 @@ main (int argc, char **argv)
   if (write_permuted_output (head_lines, line, lo_input, permutation) != 0)
     error (EXIT_FAILURE, errno, _("write error"));
 
+#ifdef lint
+  free (permutation);
+  randint_all_free (randint_source);
+  if (input_lines)
+    {
+      free (input_lines[0]);
+      free (input_lines);
+    }
+#endif
+
   return EXIT_SUCCESS;
 }
